<?php

/**
 * Implements hook_field_formatter_info().
 */
function token_formatters_field_formatter_info() {
  $info['token_formatters_entity_reference'] = array(
    'label' => t('Tokenized text'),
    'description' => t('Display tokenized text as an optional link with tokenized destination.'),
    'field types' => array('entityreference'),
    'settings' => array(
      'text' => '[entity:label]',
      'link' => '[entity:url]',
    ),
  );

  return $info;
}

/**
 * Implements hook_field_formatter_settings_form().
 */
function token_formatters_field_formatter_settings_form($field, $instance, $view_mode, $form, &$form_state) {
  $element = array();
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];

  if ($display['type'] == 'token_formatters_entity_reference') {
    $token_type = token_get_entity_mapping('entity', $field['settings']['target_type']);
    $element['text'] = array(
      '#type' => 'textfield',
      '#title' => t('Text to output.'),
      '#default_value' => $settings['text'],
      '#description' => t('This field accepts tokens.'),
      '#element_validate' => array('token_element_validate'),
      '#token_types' => array($token_type, 'entity'),
      '#required' => TRUE,
    );
    $element['link'] = array(
      '#type' => 'textfield',
      '#title' => t('Link destination'),
      '#description' => t('Leave blank to output the text only not as a link. This field accepts tokens.'),
      '#default_value' => $settings['link'],
      '#element_validate' => array('token_element_validate'),
      '#token_types' => array($token_type, 'entity'),
    );
  }

  return $element;
}

/**
 * Implements hook_field_formatter_settings_summary().
 */
function token_formatters_field_formatter_settings_summary($field, $instance, $view_mode) {
  $summary = array();
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];

  if ($display['type'] == 'token_formatters_entity_reference') {
    $summary[] = t('Text: &text', array('&text' => trim($settings['text'])));
    if ($link = trim($settings['link'])) {
      $summary[] = t('Linked to %link', array('%link' => $link));
    }
  }

  return implode('<br />', $summary);
}

/**
 * Implements hook_field_formatter_prepare_view().
 */
function token_formatters_field_formatter_prepare_view($entity_type, $entities, $field, $instances, $langcode, &$items, $displays) {
  if (module_hook($field['module'], 'field_formatter_prepare_view')) {
    $callback = $field['module'] . '_field_formatter_prepare_view';
    return $callback($entity_type, $entities, $field, $instances, $langcode, $items, $displays);
  }
}

/**
 * Implements hook_field_formatter_view().
 */
function token_formatters_field_formatter_view($entity_type, $entity, $field, $instance, $langcode, $items, $display) {
  $element = array();
  $settings = $display['settings'];

  if ($display['type'] == 'token_formatters_entity_reference') {
    foreach ($items as $delta => $item) {
      // Build the token data and options.
      $token_data = array(
        $field['settings']['target_type'] => $item['entity'],
        'entity' => $item['entity'],
        'entity_type' => $field['settings']['target_type'],
      );
      $token_options = array('santize' => FALSE, 'clear' => TRUE);

      $text = token_replace(trim($settings['text']), $token_data, $token_options);
      if ($link = token_replace(trim($settings['link']), $token_data, $token_options)) {
        $element[$delta] = array(
          '#type' => 'link',
          '#title' => $text,
          '#href' => $link,
        );
      }
      else {
        $element[$delta] = array('#markup' => check_plain($text));
      }
    }
  }

  return $element;
}
